//=========================================================================
// Arcade System Object
//=========================================================================
if(!isObject(ArcadeSystem))
{
	new scriptGroup(ArcadeSystem);
}
//=========================================================================
// Functions
//=========================================================================
function registerArcadeGame(%name, %gui, %bitmap, %description)
{
	if(strReplace(%name, " ", "") $= "")
	{
		error("Invalid name.");

		return;
	}

	if(isRegisteredArcadeGame(%name))
	{
		error("Name taken.");

		return;
	}

	if(!isObject(%gui))
	{
		error("GUI does not exist.");

		return;
	}

	if(%gui.getClassName() !$= "GuiControl")
	{
		error("Invalid GUI.");

		return;
	}

	if(!isFile(%description))
	{
		error("Description file does not exist.");

		return;
	}

	if(fileExt(%description) !$= ".txt")
	{
		error("Invalid description file.");

		return;
	}

	if(!isFile(%bitmap))
	{
		%bitmap = "Add-Ons/Support_Arcade/images/questionMark.png";

		warn("Bitmap file does not exist. Using default bitmap.");
	}

	if(fileExt(%bitmap) !$= ".png" && fileExt(%bitmap) !$= ".jpg" && fileExt(%bitmap) !$= "jpeg")
	{
		%bitmap = "Add-Ons/Support_Arcade/images/questionMark.png";

		warn("Invalid bitmap file, must be png, jpg, or jpeg. Using Default bitmap.");
	}

	%obj = new scriptObject()
	{
		name = %name;
		gui = %gui;
		bitmap = %bitmap;
		description = %description;
	};

	ArcadeSystem.add(%obj);

	if(ArcadeGui_Description.getValue() $= "<color:686868><font:Verdana Bold:15>Arcade<br><br>Author: jes00<br><br>This add on is just a base to play arcade games. It comes with no games and is mean\'t for people to add their own games to.")
	{
		ArcadeGui_Description.setText("");
	}

	if(ArcadeGui.isAwake())
	{
		ArcadeGui_Scroll.deleteAll();
		ArcadeGui.onWake();
	}

	%count = -1;

	for(%i = 0; %i < ArcadeSystem.getCount(); %i++)
	{
		%word[%count++] = ArcadeSystem.getObject(%i).name;
	}

	%sorter = new GuiTextListCtrl();

	for(%i = 0; %i < %count; %i++)
	{
		%sorter.addRow(%i, %word[%i]);
	}

	%sorter.sort(0, 0);

	for(%i = 0; %i < %sorter.rowCount(); %i++)
	{
		%sorted[%i] = %sorter.getRowText(%i);
	}

	for(%i = 0; %i < %sorter.rowCount(); %i++)
	{
		for(%ii = 0; %ii < ArcadeSystem.getCount(); %ii++)
		{
			if(ArcadeSystem.getObject(%ii).name $= %sorted[%i])
			{
				ArcadeSystem.bringToFront(ArcadeSystem.getObject(%ii));

				//echo("\c2Brought" SPC ArcadeSystem.getObject(%ii).name SPC "To front.");

				break;
			}
		}
	}

	%sorter.delete();
}

function updateArcadeGame(%name, %gui, %bitmap, %description)
{
	if(strReplace(%name, " ", "") $= "")
	{
		error("Invalid name.");

		return;
	}

	if(!isObject(%obj = isRegisteredArcadeGame(%name)))
	{
		error("Game not found.");

		return;
	}

	if(!isObject(%gui))
	{
		error("GUI does not exist.");

		return;
	}

	if(%gui.getClassName() !$= "GuiControl")
	{
		error("Invalid GUI.");

		return;
	}

	if(!isFile(%description))
	{
		error("Description file does not exist.");

		return;
	}

	if(fileExt(%description) !$= ".txt")
	{
		error("Invalid description file.");

		return;
	}

	if(!isFile(%bitmap))
	{
		%bitmap = "Add-Ons/Support_Arcade/images/questionMark.png";

		error("Bitmap file does not exist. Using default bitmap.");
	}

	if(fileExt(%bitmap) !$= ".png" && fileExt(%bitmap) !$= ".jpg" && fileExt(%bitmap) !$= "jpeg")
	{
		%bitmap = "Add-Ons/Support_Arcade/images/questionMark.png";

		warn("Invalid bitmap file, must be png, jpg, or jpeg. Using Default bitmap.");
	}

	%obj.delete();

	%obj = new scriptObject()
	{
		name = %name;
		gui = %gui;
		bitmap = %bitmap;
		description = %description;
	};

	ArcadeSystem.add(%obj);

	if(ArcadeGui.isAwake())
	{
		ArcadeGui_Scroll.deleteAll();
		ArcadeGui.onWake();
	}

	%count = -1;

	for(%i = 0; %i < ArcadeSystem.getCount(); %i++)
	{
		%word[%count++] = ArcadeSystem.getObject(%i).name;
	}

	%sorter = new GuiTextListCtrl();

	for(%i = 0; %i < %count; %i++)
	{
		%sorter.addRow(%i, %word[%i]);
	}

	%sorter.sort(0, 0);

	for(%i = 0; %i < %sorter.rowCount(); %i++)
	{
		%sorted[%i] = %sorter.getRowText(%i);
	}

	for(%i = 0; %i < %sorter.rowCount(); %i++)
	{
		for(%ii = 0; %ii < ArcadeSystem.getCount(); %ii++)
		{
			if(ArcadeSystem.getObject(%ii).name $= %sorted[%i])
			{
				ArcadeSystem.getObject(%ii).bringToFront();

				break;
			}
		}
	}

	%sorter.delete();
}

function unregisterArcadeGame(%name)
{
	if(!isObject(%obj = isRegisteredArcadeGame(%name)))
	{
		error("Arcade game not found:" SPC %name @ ". Cannot unregister game.");

		return;
	}

	%obj.delete();

	warn("Arcade game:" SPC %name @ ". Has been unregistered.");

	if(ArcadeGui.isAwake())
	{
		ArcadeGui_Scroll.deleteAll();
		ArcadeGui.onWake();
	}

	%count = -1;

	for(%i = 0; %i < ArcadeSystem.getCount(); %i++)
	{
		%word[%count++] = ArcadeSystem.getObject(%i).name;
	}

	%sorter = new GuiTextListCtrl();

	for(%i = 0; %i < %count; %i++)
	{
		%sorter.addRow(%i, %word[%i]);
	}

	%sorter.sort(0, 0);

	for(%i = 0; %i < %sorter.rowCount(); %i++)
	{
		%sorted[%i] = %sorter.getRowText(%i);
	}

	for(%i = 0; %i < %sorter.rowCount(); %i++)
	{
		for(%ii = 0; %ii < ArcadeSystem.getCount(); %ii++)
		{
			if(ArcadeSystem.getObject(%ii).name $= %sorted[%i])
			{
				ArcadeSystem.getObject(%ii).bringToFront();

				break;
			}
		}
	}

	%sorter.delete();
}

function isRegisteredArcadeGame(%name)
{
	if(strReplace(%name, " ", "") $= "")
	{
		return false;
	}

	%count = ArcadeSystem.getCount();

	if(!%count)
	{
		return false;
	}

	for(%i = 0; %i < %count; %i++)
	{
		%obj = ArcadeSystem.getObject(%i);

		if(%obj.name $= %name)
		{
			return %obj;
		}
	}

	return false;
}

function ArcadeGui::onWake(%gui)
{
	if($EditArcadeGUI)
	{
		return;
	}

	%count = ArcadeSystem.getCount();

	if(%count)
	{
		ArcadeSystem.selectedGame = "";
		ArcadeGui_Description.setText("");
	}

	for(%i = 0; %i < %count; %i++)
	{
		%obj = ArcadeSystem.getObject(%i);

		if(!ArcadeGui_Scroll.getCount())
		{
			%position = "1 1";
		}

		else
		{
			%position = "1" SPC restWords(ArcadeGui_Scroll.getObject(ArcadeGui_Scroll.getCount() - 1).position) + 80;
		}

		%this = new GuiBitmapBorderCtrl()
		{
			profile = "ArcadeBorderProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = %position;
			extent = "236 70";
			minExtent = "8 2";
			enabled = "1";
			visible = "1";
			clipToParent = "1";

			new GuiSwatchCtrl()
			{
				profile = "GuiDefaultProfile";
				horizSizing = "right";
				vertSizing = "bottom";
				position = "1 1";
				extent = "234 68";
				minExtent = "8 2";
				enabled = "1";
				visible = "1";
				clipToParent = "1";
				color = "0 0 0 30";
			};

			new GuiBitmapCtrl()
			{
				profile = "GuiDefaultProfile";
				horizSizing = "right";
				vertSizing = "bottom";
				position = "5 5";
				extent = "60 60";
				minExtent = "8 2";
				enabled = "1";
				visible = "1";
				clipToParent = "1";
				bitmap = "Add-Ons/Support_Arcade/images/questionMark";
				wrap = "0";
				lockAspectRatio = "0";
				alignLeft = "0";
				alignTop = "0";
				overflowImage = "0";
				keepCached = "0";
				mColor = "255 255 255 255";
				mMultiply = "0";
			};

			new GuiBitmapCtrl()
			{
				profile = "GuiDefaultProfile";
				horizSizing = "right";
				vertSizing = "bottom";
				position = "5 5";
				extent = "60 60";
				minExtent = "8 2";
				enabled = "1";
				visible = "1";
				clipToParent = "1";
				bitmap = %obj.bitmap;
				wrap = "0";
				lockAspectRatio = "0";
				alignLeft = "0";
				alignTop = "0";
				overflowImage = "0";
				keepCached = "0";
				mColor = "255 255 255 255";
				mMultiply = "0";
			};

			new GuiBitmapButtonCtrl()
			{
				profile = "ArcadeButtonLargeProfile";
				horizSizing = "right";
				vertSizing = "bottom";
				position = "0 0";
				extent = "236 70";
				minExtent = "8 2";
				enabled = "1";
				visible = "1";
				clipToParent = "1";
				command = "ArcadeGui.selectGame(\"" @ %obj.name @ "\");";
				text = "          " @ %obj.name;
				groupNum = "-1";
				buttonType = "PushButton";
				bitmap = "base/client/ui/btnBlank";
				lockAspectRatio = "0";
				alignLeft = "0";
				alignTop = "0";
				overflowImage = "0";
				mKeepCached = "0";
				mColor = "255 255 255 255";
			};
		};

		ArcadeGui_Scroll.add(%this);

		if(ArcadeGui_Scroll.getCount() >= 4)
		{
			ArcadeGui_Scroll.resize(firstWord(ArcadeGui_Scroll.getPosition()), restWords(ArcadeGui_Scroll.getPosition()), 243, restWords(ArcadeGui_Scroll.getObject(ArcadeGui_Scroll.getCount() - 1).position) + restWords(ArcadeGui_Scroll.getObject(ArcadeGui_Scroll.getCount() - 1).extent) + 12);
		}
	}
}

function ArcadeGui::selectGame(%gui, %uiName)
{
	if(!isObject(%obj = isRegisteredArcadeGame(%uiName)))
	{
		return;
	}

	%file = new fileObject();
	%file.openForRead(%obj.description);

	ArcadeGui_Description.setText("");

	while(!%file.isEOF())
	{
		%line = %file.readLine();

		if(ArcadeGui_Description.getValue() $= "")
		{
			ArcadeGui_Description.setText("<color:686868><font:Verdana Bold:15>" @ %line);
		}

		else
		{
			ArcadeGui_Description.setText(ArcadeGui_Description.getValue() NL %line);
		}
	}

	%file.close();
	%file.delete();

	ArcadeSystem.selectedGame = %uiName;
}

function getSelectedArcadeGame(%bool)
{
	if(!ArcadeGui.isAwake())
	{
		return "";
	}

	if(!%bool)
	{
		return ArcadeSystem.selectedGame;
	}

	else
	{
		%count = ArcadeSystem.getCount();

		for(%i = 0; %i < %count; %i++)
		{
			%obj = ArcadeSystem.getObject(%i);

			if(%obj.name $= ArcadeSystem.selectedGame)
			{
				return %obj;
			}
		}

		return "";
	}
}

function ArcadeGui::playGame(%gui)
{
	if(!isObject(%obj = getSelectedArcadeGame(1)))
	{
		return;
	}

	canvas.popDialog(%gui);
	canvas.pushDialog(%obj.gui);
}

function ArcadeGui::openSettings(%gui)
{
	
}

function editArcadeGUI(%bool)
{
	$EditArcadeGUI = %bool;

	if(%bool)
	{
		ArcadeGui_Description.setText("<color:686868><font:Verdana Bold:15>Arcade<br><br>Author: jes00<br><br>This add on is just a base to play arcade games. It comes with no games and is mean\'t for people to add their own games to.");
	}

	else
	{
		if(ArcadeSystem.getCount() && ArcadeGui_Description.getValue() $= "<color:686868><font:Verdana Bold:15>Arcade<br><br>Author: jes00<br><br>This add on is just a base to play arcade games. It comes with no games and is mean\'t for people to add their own games to.")
		{
			ArcadeGui_Description.setText("");
		}
	}
}
//=========================================================================
// Package
//=========================================================================
package Arcade
{
	function canvas::popDialog(%this, %gui)
	{
		parent::popDialog(%this, %gui);

		if(%gui $= "ArcadeGui")
		{
			if(ArcadeGui_Scroll.getCount())
			{
				ArcadeGui_Scroll.deleteAll();

				//This function delete's all of the
				//scroll's children

				ArcadeGui_Scroll.resize(firstWord(ArcadeGui_Scroll.getPosition()), restWords(ArcadeGui_Scroll.getPosition()), 243, 283);
			}
		}
	}

	function GuiBitmapButtonCtrl::setText(%obj, %text)
	{
		if(%obj.profile $= "ArcadeButtonLargeProfile")
		{
			if(getSubStr(%text, 0, 10) !$= "          ")
			{
				%text = "          " @ %text;
			}
		}

		parent::setText(%obj, %text);
	}

	function GuiEditorInspectFields::apply(%obj, %name)
	{
		if(InspectStaticProfile0_27.getValue() $= "ArcadeButtonLargeProfile")
		{
			if(getSubStr(InspectStatictext0_11.getValue(), 0, 10) !$= "          ")
			{
				InspectStatictext0_11.setText("          " @ InspectStatictext0_11.getValue());
			}
		}

		parent::apply(%obj, %name);
	}

	function onExit()
	{
		echo("Exporting Arcade high scores.");

		saveArcadeHighScores();

		parent::onExit();
	}
};
activatePackage(Arcade);
//=========================================================================
// Profiles
//=========================================================================
if(isObject(ArcadeWindowProfile))
{
	ArcadeWindowProfile.delete();
}

new GuiControlProfile(ArcadeWindowProfile)
{
	opaque = 1;
	border = 2;
	fontType = "Impact";
	fontSize = 18;
	fontColors[0] = "255 255 255 255";
	fontColors[1] = "255 255 255 255";
	fontColor = "255 255 255 255";
	fontColorHL = "255 255 255 255";
	justify = "center";
	textOffset = "5 2";
	bitmap = "Add-Ons/Support_Arcade/images/ArcadeWindow";

	text = "";
	hasBitmapArray = 1;
};

if(isObject(ArcadeBorderProfile))
{
	ArcadeBorderProfile.delete();
}
new GuiControlProfile(ArcadeBorderProfile : GuiBitmapBorderProfile)
{
	bitmap = "Add-ons/Support_Arcade/images/contentArray";
};

if(isObject(ArcadeScrollProfile))
{
	ArcadeScrollProfile.delete();
}

new GuiControlProfile(ArcadeScrollProfile)
{
	fontType = "Book Antiqua";
	fontSize = 22;
	justify = center;
	fontColor = "0 0 0";
	fontColorHL = "130 130 130";
	fontColorNA = "255 0 0";
	fontColors[0] = "0 0 0";
	fontColors[1] = "0 255 0";
	fontColors[2] = "0 0 255";
	fontColors[3] = "255 255 0";
	hasBitmapArray = 1;
	bitmap = "Add-ons/Support_Arcade/images/scrollArray";
};

if(isObject(ArcadeButtonProfile))
{
	ArcadeButtonProfile.delete();
}

new GuiControlProfile(ArcadeButtonProfile : GuiDefaultProfile)
{
	fontColor = "68 68 68 255";
	fontSize = 15;
	fontType = "Verdana Bold";
	justify = "center";
};

if(isObject(ArcadeButtonLargeProfile))
{
	ArcadeButtonLargeProfile.delete();
}

if(isObject(ArcadeButtonProfile))
{
	new GuiControlProfile(ArcadeButtonLargeProfile : ArcadeButtonProfile)
	{
		fontSize = 25;
		justify = "left";
		textOffset = "65 0";
	};
}
//=========================================================================
// Misc
//=========================================================================
function scrunchGuiCtrl(%obj, %offsetX, %offsetY, %bool)
{
	%count = %obj.getCount();

	if(!%count)
	{
		return;
	}

	%xExtent = 0;
	%yExtent = 0;

	for(%i = 0; %i < %count; %i++)
	{
		%child = %obj.getObject(%i);
		%x = firstWord(%child.getExtent()) + firstWord(%child.getPosition());
		%y = restWords(%child.getExtent()) + restWords(%child.getPosition());

		if(%x > %xExtent)
		{
			%xExtent = %x;
		}

		if(%y > %yExtent)
		{
			%yExtent = %y;
		}
	}

	%xExtent = %xExtent + %offsetX;
	%yExtent = %yExtent + %offsetY;

	%obj.resize(firstWord(%obj.getPosition()), restWords(%obj.getPosition()), %xExtent, %yExtent);
}

function addBind(%division, %name, %command)
{
	for(%i=0;%i<$remapCount;%i++)
	{
		if($remapDivision[%i] $= %division)
		{
			%foundDiv = 1;
			continue;
		}
		if(%foundDiv && $remapDivision[%i] !$= "")
		{
			%position = %i;
			break;
		}
	}
	if(!%foundDiv)
	{
		error("Division not found: " @ %division);
		return;
	}
	if(!%position)
	{
		$remapName[$remapCount] = %name;
		$remapCmd[$remapCount] = %command;
		$remapCount++;
		return;
	}
	for(%i=$remapCount;%i>%position;%i--)
	{
		$remapDivision[%i] = $remapDivision[%i - 1];
		$remapName[%i] = $remapName[%i - 1];
		$remapCmd[%i] = $remapCmd[%i - 1];
	}
	$remapDivision[%position] = "";
	$remapName[%position] = %name;
	$remapCmd[%position] = %command;
	$remapCount++;
}

if(!$Arcade::AddedBind)
{
	$Arcade::AddedBind = true;

	addBind("Gui", "Toggle Arcade", "toggleArcade");
}

function toggleArcade(%bool)
{
	if(!%bool)
	{
		return;
	}

	if(!ArcadeGui.isAwake())
	{
		if(%bool == 2)
		{
			%s = alxCreateSource(AudioChannel0, "base/data/sound/clickPlant.wav");
			alxPlay(%s);
		}

		canvas.pushDialog(ArcadeGui);
	}

	else
	{
		canvas.popDialog(ArcadeGui);
	}
}

if(isObject(MM_ArcadeButton))
{
	MM_ArcadeButton.delete();
}

new GuiBitmapButtonCtrl(MM_ArcadeButton)
{
	profile = "BlockButtonProfile";
	horizSizing = "relative";
	vertSizing = "relative";
	position = "500 385";
	extent = "140 40";
	minExtent = "8 2";
	enabled = 1;
	visible = 1;
	clipToParent = 1;
	command = "toggleArcade(2);";
	text = " ";
	groupNum = -1;
	buttonType = "PushButton";
	bitmap = "Add-Ons/Support_Arcade/images/ArcadeButton";
	mColor = "255 255 255 255";
};

MainMenuButtonsGui.add(MM_ArcadeButton);

function saveArcadeHighScores()
{
	export("$ArcadeHighScore::*", "config/client/ArcadeHighScores.cs");
}

if(isFile("config/client/ArcadeHighScores.cs"))
{
	exec("config/client/ArcadeHighScores.cs");
}

if(isObject(ArcadeGui))
{
	ArcadeGui.delete();
}

exec("./ArcadeGui.gui");